# frozen_string_literal: true

require 'git'
require 'json'
require 'date'
require '/data_importer/lib/importers/github_repo.rb'

class TrickestPocCveImporter < GithubRepo

  def initialize
    super(repo_url = 'https://github.com/trickest/cve.git', repo_path = '/data_importer/data/trickest_cve')
  end

  def html_to_hash(html)
    data_hash = {}
    doc = Nokogiri::HTML5.parse(html)
    h3_nodes = doc.xpath('//h3')
    h3_keys = doc.xpath('//h3').map { |n| n.children.first.text }
    h4_keys = doc.xpath('//h4').map { |n| n.children.first.text }
    data_hash_keys = (h3_keys + h4_keys).flatten

    # cve id is always the first url in the markdown doc
    cve_url = doc.xpath('//h3/a').attribute('href').value
    cve_id = h3_keys.first

    p_text = doc.xpath('//p').map(&:text)
    links_for_poc = doc.xpath('//p/a').map(&:values).flatten

    data_hash['cve_id'] = cve_id
    data_hash['cve_url'] = cve_url
    # p_text[0] is always an ' '.
    data_hash['description'] = p_text[1]

    data_hash['poc_links'] = links_for_poc
    data_hash
  end

  def list_mds_for_year(year)
    year_fp = "#{repo_path}/#{year}"
    Dir["#{year_fp}/*.md"]
  end

  def read_mds_for_year(year)
    filenames = list_mds_for_year(year)
    filenames.map { |filename| read_markdown(filename) }
  end

  def read_all_mds
    (1999..Date.today.year).map do |year|
      read_mds_for_year(year.to_s)
    end
  end

  def cve_attrs_from_item(json)
    cve_attrs = {}
    cve_attrs[:cve_id] = json['cve_id']
    cve_attrs[:cve_url] = json['cve_url']
    cve_attrs[:description] = json['description']
    cve_attrs[:poc_links] = json['poc_links']
    cve_attrs
  end

  def cves_for_year(year)
    htmls = read_mds_for_year(year)
    htmls.map do |html|
      data_hash = html_to_hash(html)
      cve_attrs_from_item(data_hash)
    end
  end

  def import
    pull_or_clone
    puts "Now starting import for #{repo_url}."
    puts '----------' * 12
    (1999..Date.today.year).map do |year|
      cves_from_markdown = cves_for_year(year)
      puts "Importing any new CVEs from #{year}"
      TrickestPocCve.upsert_all(cves_from_markdown, unique_by: :cve_id)
    end
  end
end
